﻿Shader "CoreMission/CardParallax2"
{
    Properties
    {
        _Color("Color", Color) = (1,1,1,1)
        _TopMap("Top texture", 2D) = "white" {}
        _BottomMap("Bottom texture", 2D) = "white" {}
        _NormalMap("Normal texture", 2D) = "white" {}
        _EmissiveMap("Emissive texture", 2D) = "black" {}
        _EmissionIntensity("Emission intensity", Range(0,10)) = 1.1
        _Glossiness("Smoothness", Range(0,1)) = 0.5
        _Metallic("Metallic", Range(0,1)) = 0.0
        _OffsetScale("Offset scale", float) = 0
        _AlphaTestThreshold("Alpha threshold", float) = 0.1
        _BottomCardScale("Bottom Card Scale", float) = 1.0
    }

    SubShader
    {
        Tags {"Queue" = "Transparent" "RenderType"="Transparent"}
        LOD 200
        ZWrite Off
        
        CGPROGRAM
        // Physically based Standard lighting model, and enable shadows on all light types
        #pragma surface surf Standard fullforwardshadows vertex:vert alpha

        // Use shader model 3.0 target, to get nicer looking lighting
        #pragma target 3.0

        sampler2D _TopMap;
        sampler2D _BottomMap;
        sampler2D _EmissiveMap;
        sampler2D _NormalMap;

        struct Input {
            float2 uv_TopMap;
            float2 uv_BottomMap;
            float3 vs_normal;
        };

        half _EmissionIntensity;
        half _Glossiness;
        half _Metallic;
        fixed4 _Color;
        float _OffsetScale;
        float _BottomCardScale;
        float _AlphaTestThreshold;

        void vert(inout appdata_full v, out Input o)
        {
            UNITY_INITIALIZE_OUTPUT(Input,o);
            // This give Annoying warning, but I couldn't found way to transform Normal right
            // UnityObjectToViewPos exist, I know, but where is UnityObjectToViewDir?
            o.vs_normal = normalize(mul((float3x3)UNITY_MATRIX_MV, v.normal));
        }

        void surf(Input IN, inout SurfaceOutputStandard o) 
        {
            // alpha, emissive, roughness, metallness
            float4 arm = tex2D(_EmissiveMap, IN.uv_TopMap);
            clip(arm.r - _AlphaTestThreshold);

            fixed4 topColor = tex2D(_TopMap, IN.uv_TopMap) * _Color;

            float2 offset_uv = IN.uv_BottomMap + _OffsetScale * IN.vs_normal.xy;
            offset_uv -= float2(0.5, 0.5);
            offset_uv *= _BottomCardScale;
            offset_uv += float2(0.5, 0.5);
            // todo: without MIP
            //float4 bottomColor = tex2D(_BottomMap, offset_uv);

            float4 bottomColor = tex2Dlod(_BottomMap, float4(offset_uv, 0, /* 9 determines biggest MIP level for opaque zone */ 9 * topColor.a ));


            o.Emission = _EmissionIntensity * topColor.rgb;
            
            o.Albedo = lerp(bottomColor, topColor.rgb, topColor.a);

            // todo: show mips animation
            //o.Albedo = tex2Dlod(_BottomMap, float4(IN.uv_BottomMap, 0, /* 8 determines biggest MIP level for opaque zone */ 8 * sin(_Time.y * 4)));
            
            // Metallic and smoothness come from slider variables

            o.Normal = tex2D(_NormalMap, IN.uv_TopMap);
            o.Metallic = arm.b;
            o.Smoothness = 1.0 - arm.g;
            o.Alpha = arm.a;
        }
        ENDCG
    }
    FallBack "Diffuse"
}