﻿// Upgrade NOTE: replaced 'mul(UNITY_MATRIX_MVP,*)' with 'UnityObjectToClipPos(*)'

Shader "CoreMission/LayeredParallax"
{
	Properties
    {
		_Color("Color", Color) = (1,1,1,1)
		_MainTex("Albedo (RGB)", 2D) = "white" {}
		_Glossiness("Smoothness", Range(0,1)) = 0.5
		_Metallic("Metallic", Range(0,1)) = 0.0

		_ParallaxMap("Parallax map", 2D) = "white" {}
		_Iterations("Iterations", float) = 5
		_OffsetScale("Offset scale", float) = 0
	}

	SubShader
    {
		Tags { "RenderType" = "Opaque" "DisableBatching" = "True"}
		LOD 200

		CGPROGRAM
		// Physically based Standard lighting model, and enable shadows on all light types
		#pragma surface surf Standard fullforwardshadows vertex:vert

		// Use shader model 3.0 target, to get nicer looking lighting
		#pragma target 3.0

		sampler2D _MainTex;
		sampler2D _ParallaxMap;

		struct Input {
			float2 uv_MainTex;
			float3 view_ts;
		};

		half _Glossiness;
		half _Metallic;
		fixed4 _Color;
		float _OffsetScale;
		float _Iterations;

		void vert(inout appdata_full v, out Input o) {
			UNITY_INITIALIZE_OUTPUT(Input,o);
            float3 camera_ls  = mul(unity_WorldToObject, float4(_WorldSpaceCameraPos, 1.0)).xyz;
            float3 viewDir_ls = v.vertex.xyz - camera_ls;
            float tangentSign = v.tangent.w * unity_WorldTransformParams.w;
            float3 bitangent = cross(v.normal.xyz, v.tangent.xyz) * tangentSign;
            o.view_ts = float3(
                dot(viewDir_ls, v.tangent.xyz),
                dot(viewDir_ls, bitangent.xyz),
                dot(viewDir_ls, v.normal.xyz)
            );
		}

		void surf(Input IN, inout SurfaceOutputStandard o) {
			float3 parallax = 0;

			for (int j = 0; j < _Iterations; j++) {
				float ratio = (float)j / _Iterations;
				float2 uv_offset = lerp(0, _OffsetScale, ratio) * normalize(IN.view_ts).xy;
				parallax += tex2D(_ParallaxMap, IN.uv_MainTex + uv_offset) * lerp(1, 0, ratio);
			}
			parallax /= _Iterations;
			fixed4 c = tex2D(_MainTex, IN.uv_MainTex) * _Color;
			o.Albedo = c.rgb + parallax;

			o.Metallic = _Metallic;
			o.Smoothness = _Glossiness;
			o.Alpha = c.a;
		}
		ENDCG
	}
	FallBack "Diffuse"
}