﻿// Upgrade NOTE: replaced 'mul(UNITY_MATRIX_MVP,*)' with 'UnityObjectToClipPos(*)'

Shader "CoreMission/LayeredParallaxColor"
{
	Properties
    {
		_Color1("Color 1", Color) = (1,1,1,1)
		_Intensity1("Intensity 1", Range(0,10)) = 5

		_Color2("Color 2", Color) = (1,1,1,1)
		_Intensity2("Intensity 2", Range(0,10)) = 5

		_MainTex("Albedo (RGB)", 2D) = "white" {}
		_Glossiness("Smoothness", Range(0,1)) = 0.5
		_Metallic("Metallic", Range(0,1)) = 0.0

		_Iterations("Iterations", float) = 5
		_OffsetScale("Offset scale", float) = 0
	}

	SubShader
    {
		Tags { "RenderType" = "Opaque" "DisableBatching" = "True"}
		LOD 200

		CGPROGRAM
		// Physically based Standard lighting model, and enable shadows on all light types
		#pragma surface surf Standard fullforwardshadows vertex:vert

		// Use shader model 3.0 target, to get nicer looking lighting
		#pragma target 3.0

		sampler2D _MainTex;

		struct Input {
			float2 uv_MainTex;
			float3 view_dir_ts;
		};

		half _Glossiness;
		half _Metallic;
		float4 _Color1;
		float _Intensity1;
		float4 _Color2;
		float _Intensity2;
		float _OffsetScale;
		float _Iterations;

		void vert(inout appdata_full v, out Input o) {
			UNITY_INITIALIZE_OUTPUT(Input,o);
            float3 camera_ls  = mul(unity_WorldToObject, float4(_WorldSpaceCameraPos, 1.0)).xyz;
            float3 viewDir_ls = v.vertex.xyz - camera_ls;
            float tangentSign = v.tangent.w * unity_WorldTransformParams.w;
            float3 bitangent = cross(v.normal.xyz, v.tangent.xyz) * tangentSign;
            o.view_dir_ts = float3(
                dot(viewDir_ls, v.tangent.xyz),
                dot(viewDir_ls, bitangent.xyz),
                dot(viewDir_ls, v.normal.xyz)
            );
		}

		void surf(Input IN, inout SurfaceOutputStandard o) {
			float3 parallax = float3(0, 0, 0);

			float3 color1 = _Intensity1 * _Color1;
			float3 color2 = _Intensity2 * _Color2;

			for (int j = 0; j < _Iterations; j++) {
				float ratio = (float)j / _Iterations;
				float2 uv_offset = lerp(0, _OffsetScale, ratio) * normalize(IN.view_dir_ts).xy;
				float3 col = lerp(color1, color2, ratio);
				parallax += col * tex2D(_MainTex, IN.uv_MainTex + uv_offset);// * lerp(1, 0, ratio);
			}
			parallax /= _Iterations;
			
			fixed4 c = tex2D(_MainTex, IN.uv_MainTex);
			o.Albedo = parallax;

			o.Metallic = _Metallic;
			o.Smoothness = _Glossiness;
			o.Alpha = c.a;
		}
		ENDCG
	}
	FallBack "Diffuse"
}