﻿Shader "CoreMission/Portal"
{
    Properties
    {
        _NoiseMap ("Noise Map", 2D) = "white" {}
        _ColorMap ("Color Map", 2D) = "white" {}
        _Speed("Speed", Range (-10.0, 10.0)) = 0.0
        _NoiseImpact("Noise Impact", Range (0.0, 1.0)) = 0.1
        _EdgeNoiseImpact("Edge Noise Impact", Range (0.0, 1.0)) = 0.1
        _PolarDistortion("Polar Distortion Parameter", Range (0.0, 1.0)) = 0.0
        _GridColor("Grid color", Color) = (0,1,0,1)
    }
    SubShader
    {
        Tags { "RenderType"="Opaque" }
        LOD 100

        Pass
        {
            CGPROGRAM
            #pragma enable_d3d11_debug_symbols
            #pragma vertex vert
            #pragma fragment frag

            #include "UnityCG.cginc"

            struct appdata
            {
                float4 vertex : POSITION;
                float2 uv : TEXCOORD0;
            };

            struct v2f
            {
                float2 uv : TEXCOORD0;
                float4 vertex : SV_POSITION;
            };

            sampler2D _NoiseMap;
            sampler2D _ColorMap;
            float4 _NoiseMap_ST;
            float4 _GridColor;
            float _Speed;
            float _NoiseImpact;
            float _EdgeNoiseImpact;
            float _PolarDistortion;

            #define M_2PI 6.283185307
            
            v2f vert (appdata v)
            {
                v2f o;
                o.vertex = UnityObjectToClipPos(v.vertex);
                o.uv = TRANSFORM_TEX(v.uv, _NoiseMap);
                return o;
            }
/*
float4 frag (v2f i) : SV_Target
{
    // Original UV output
    //return float4(i.uv, 0, 1);

    i.uv -= float2(0.5, 0.5);
    float2 uv = i.uv;

    // Polar UV
    float2 p = i.uv;
    float a = atan2(p.y ,p.x);
    uv = float2(length(p), (a / M_2PI));

    return float4(uv, 0, 1);
}
*/
            float4 frag (v2f i) : SV_Target
            {
                // Original UV output
                //return float4(i.uv, 0, 1);

                i.uv -= float2(0.5, 0.5);
                float2 uv = i.uv;

                // Polar UV
                float2 p = i.uv;
                float a = atan2(p.y ,p.x);
                uv = float2(length(p), (a / M_2PI));
                
                // This lerp is to visualise plane wrapping
                uv = lerp(i.uv, uv, _PolarDistortion);
                
                float noise = tex2D(_NoiseMap, i.uv).x;
                float offset = _Time.x * _Speed;
                
                // Apply Noise and Offset
                uv += float2(offset, offset + noise * _NoiseImpact);
                
                // Transformed UV output
                //return float4(uv, 0, 1);

                // Color output
                float edgeNoise = tex2D(_NoiseMap, uv).x * _EdgeNoiseImpact;
                float edge = 1.0 - smoothstep(0.3, 0.4 + edgeNoise, length(i.uv));

                return edge * tex2D(_ColorMap, uv);

                // GRID
                const float cellSize = 0.1;
                const float gridSize = 0.02;
                float grid = 1.0 - step(gridSize, abs(uv.x) % cellSize);
                grid += 1.0 - step(gridSize, abs(uv.y) % cellSize);
                grid = saturate(grid);
                
                // axis
                float axis = 1.0 - step(0.5 *gridSize, abs(uv.x));
                axis += 1.0 - step(0.5 * gridSize, abs(uv.y));
                axis = saturate(axis);

                return grid * _GridColor + axis * float4(0, 1, 0, 1);
            }

            ENDCG
        }
    }
}
