﻿Shader "CoreMission/Spyral"
{
    Properties
    {
        _NoiseMap ("Noise Map", 2D) = "white" {}
        _ColorMap ("Color Map", 2D) = "white" {}
        _Speed("Speed", Range (0.0, 10.0)) = 0.0
        _DistanceMultiplier("Distance Multiplier", Range (0, 100)) = 1
        _AngleMultiplier("Angle Multiplier", Range (0, 100)) = 1
        _GridColor("Grid color", Color) = (0,1,0,1)
    }
    SubShader
    {
        Tags { "RenderType"="Opaque" }
        LOD 100

        Pass
        {
            CGPROGRAM
            #pragma vertex vert
            #pragma fragment frag

            #include "UnityCG.cginc"

            struct appdata
            {
                float4 vertex : POSITION;
                float2 uv : TEXCOORD0;
            };

            struct v2f
            {
                float2 uv : TEXCOORD0;
                float4 vertex : SV_POSITION;
            };

            sampler2D _NoiseMap;
            sampler2D _ColorMap;
            float4 _NoiseMap_ST;
            float4 _GridColor;
            float _Speed;
            uint _DistanceMultiplier;
            uint _AngleMultiplier;

            #define M_2PI 6.283185307
            
            v2f vert (appdata v)
            {
                v2f o;
                o.vertex = UnityObjectToClipPos(v.vertex);
                o.uv = TRANSFORM_TEX(v.uv, _NoiseMap);
                return o;
            }

            float4 frag (v2f i) : SV_Target
            {
                i.uv -= float2(0.5, 0.5);
                i.uv *= 2.0;
                float2 uv = i.uv;

                float d = length(uv);
                float angle = atan2(uv.y, uv.x);

                float color = sin(d * _DistanceMultiplier + angle * _AngleMultiplier - _Time.y * _Speed);
                return float4(color, color, color, 1);
            }

            ENDCG
        }
    }
}
